/*!***********************************************************************
 *************************************************************************
 * \file            SerialPorts.h
 *
 * \author          copyright 2005-2006
 *                  Kai Klenovsek
 * 
 * \date            First Step: 2005-07-27 
 *
 * \b LICENSE:
 *         This Software is free but without any warranty.
 *         You must include a hint to the copyrigtht holder 
 *         into your application if you use the DLL.
 *
 * \b NOTES:
 *
 * \todo  
 *               
 * \bug
 *         
 * CHANGE HISTORY:
 *
 *      Kai K. / 2006-01-10
 *            - Support for Comport greater 9 now.
 *
 *      Kai K. / 2005-12-15
 *            - The thread functions implemented.
 *            - Some documentation bugfixes and updates.
 *            - Change the return type of function ser_ReadString().
 *            - Dll is now usable with pure C application also.
 *
 *      Kai K. / 2005-08-25
 *            - Comport number bugfix.
 *
 *      Kai K. / 2005-08-08
 *            - Some define bugfixes.
 *
 *      Kai K. / 2005-08-01
 *            - Function ser_SetStopBits(), ser_SetDataLenght() and 
 *              ser_SetParityBit() implemented.
 *            - Fixed structur pointer problem in ser_OpenComPort() and
 *              ser_SetTimeouts().
 * 
***************************************************************************
***************************************************************************/

#ifndef SERIAL_PORTS_DLL_H
#define SERIAL_PORTS_DLL_H




/***********************************************************************
* 
* DEFINES
*  
***********************************************************************/

/* This statement is only for Dll building */
#define DLL_IM_EXPORT __declspec(dllexport)

/* Change here your import statement addicted to your compiler.
 * Otherwhise you can't import the Dll functions into your
 * application. This one is only for Visual C++ greater Version 4 */
//#define DLL_IM_EXPORT   __declspec(dllimport)



/***********************************************************************
* 
* ENUMS
*  
***********************************************************************/

/*! \brief Com ports */
enum eComPort {
    COMPORT1 = 0,
    COMPORT2,
    COMPORT3,
    COMPORT4,
    COMPORT5,
    COMPORT6,
    COMPORT7,
    COMPORT8,
    COMPORT9,
    COMPORT10,
    COMPORT11,
    COMPORT12,
    COMPORT13,
    COMPORT14,
    COMPORT15,
    COMPORT16,
    COMPORT17,
    COMPORT18,
    COMPORT19,
    COMPORT20
};

/*! \brief Baudrate */
enum eBaudRate {
    BR110    = 110,
    BR150    = 150,
    BR300    = 300,
    BR600    = 600,
    BR1200   = 1200,
    BR2400   = 2400,
    BR4800   = 4800,
    BR9600   = 9600,
    BR14400  = 14400,
    BR19200  = 19200,
    BR28800  = 28800,
    BR38400  = 38400,
    BR56000  = 56000,
    BR57600  = 57600,
    BR115200 = 115200,
    BR128000 = 128000,
    BR256000 = 256000
};

/*! \brief Stop bit setting */
enum eStopBits {
    STOP_ONE          = 0,
    STOP_ONEPOINTFIVE,
    STOP_TWO
};

/*! \brief Data length in bits */
enum eDataLenght {
    LEN_FOUR  = 4,
    LEN_FIVE  = 5,
    LEN_SIX   = 6,
    LEN_SEVEN = 7,
    LEN_EIGHT = 8,
    LEN_NINE  = 9
};

/*! \brief Parity bit settings */
enum eParityBit {
    PAR_NOONE = 0,
    PAR_ODD,
    PAR_EVEN,
    PAR_MARK,
    PAR_SPACE
};

/*! \brief DTR settings */
enum eDTRConfig {
    DTR_DISABLE   = 0,                      // Switch off DTR line.
    DTR_ENABLE,                             // Switch on DTR line.
    DTR_HANDSHAKE
};

/*! \brief RTS settings */
enum eRTSConfig {
    RTS_DISABLE   = 0,                      // Switch off RTS line.
    RTS_ENABLE,                             // Switch on RTS line.
    RTS_HANDSHAKE,
    RTS_TOGGLE
};




/***********************************************************************
* 
* TYPEDEFS
*  
***********************************************************************/

/*! \brief Comport initialization structur  */
typedef struct stPortInfo {
    eComPort        ePort;
    eBaudRate       eBaud;
    eStopBits       eStop;
    eDataLenght     eLenght;
    eParityBit      eParity;
    eDTRConfig      eDtr;
    eRTSConfig      eRts;
} tPortInfo;

tPortInfo t_info;
tPortInfo* tInfo = &t_info;


/***********************************************************************
* 
* EXTERNAL FUNCTION PROTOTYPES
*  
***********************************************************************/

// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Close Comport.
 *  \param  hPort = Handle to an open Comport. 
 *  \return Returns true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_CloseComPort( HANDLE hPort );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Return DLL copyright.
 *  \return Pointer to NULL terminated copyright string of DLL. */
extern "C" DLL_IM_EXPORT const char* ser_GetDllCopyright( void );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Return DLL version date.
 *  \return Pointer to NULL terminated version date string of DLL. */
extern "C" DLL_IM_EXPORT const char* ser_GetDllDate( void );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Return DLL name.
 *  \return Pointer to NULL terminated name string of DLL. */
extern "C" DLL_IM_EXPORT const char* ser_GetDllName( void );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Return DLL version number.
 *  \return Pointer to NULL terminated version string of DLL*/
extern "C" DLL_IM_EXPORT const char* ser_GetDllVersion( void );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Initialisation and opening of a Comport.
 *  \param  *tInfo = Pointer to comport init data structur. 
 *  \return HANDLE = Handle to an open Comport or INVALID_HANDLE_VALUE. */
extern "C" DLL_IM_EXPORT HANDLE ser_OpenComPort( tPortInfo *tInfo );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Reads a string from Comport. See also ser_ReaderThreadStart().
 *  \param  hPort         = Handle to an open Comport. 
 *  \param  *cBuffer      = Pointer to result buffer.
 *  \param  ulLenght      = Lenght to read.
 *  \return unsigned long = Number of really read bytes. */
extern "C" DLL_IM_EXPORT unsigned long ser_ReadString( HANDLE         hPort, 
                                                       char           *cBuffer, 
                                                       unsigned long  ulLenght );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Set or change Data length on Comport.
 *  \param  hPort   = Handle to an open Comport. 
 *  \param  eLenght = Data lenght.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetDataLenght( HANDLE hPort, eDataLenght eLenght );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Configuration of the DTR line.
 *  \param  hPort = Handle to an open Comport. 
 *  \param  eDtr  = DTR enum. 
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetDtrControl( HANDLE hPort, eDTRConfig eDtr );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Set or change Paritybit on Comport.
 *  \param  hPort   = Handle to an open Comport. 
 *  \param  eParity = Paritybit configuration.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetParityBit( HANDLE hPort, eParityBit eParity );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Configuration of the RTS line.
 *  \param  hPort = Handle to an open Comport. 
 *  \param  eRts  = RTS enum. 
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetRtsControl( HANDLE hPort, eRTSConfig eRts );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Set or change Comport Baudrate.
 *  \param  hPort = Handle to an open Comport.
 *  \param  eBaud = Baudrate.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetBaudRate( HANDLE hPort, eBaudRate eBaud );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Set or change Stopbit configuration.
 *  \param  hPort = Handle to an open Comport.
 *  \param  eStop = Stopbit configuration enum.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetStopBits( HANDLE hPort, eStopBits eStop );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Sets or change the Comport timeouts.
 *  \param  hPort      = Handle to an open Comport.
 *  \param  *tTimeOuts = Pointer to Timeout structur defined in WinApi.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_SetTimeouts( HANDLE hPort, COMMTIMEOUTS *tTimeOuts );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Writes data to Comport.
 *  \param  hPort    = Handle to an open Comport. 
 *  \param  *cData   = Pointer to data.
 *  \param  ulLenght = Lenght of data. Set to zero if your string is NULL terminated
 *                     and if you want to send the complete string.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_WriteString( HANDLE         hPort, 
                                               char           *cData, 
                                               unsigned long  ulLenght );
                                               
                                                
                                              

/***********************************************************************
* 
* EXTERNAL THREAD BASED FUNCTION PROTOTYPES
* READ THE DOCUMENTATION CAREFULLY BEFORE USING THE THREAD FUNCTIONS
*  
***********************************************************************/

/*! \brief Window message if a character was sent successfull */
#define ON_SER_CHAR_SEND    (WM_APP + 3)


/*! \brief Callback Prototype */
typedef void (*fpChar)( char cCharacter, LPARAM lParam );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Stops the reader thread.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_ReaderThreadExit( void );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Starts a reader thread. If the Dll receive a char it will call 
 *          the callback function OnCharEvent(). Only one reader thread 
 *          per Dll instance is possible.
 *          NOTE: If the thread is running you should avoid all access to 
 *                the port. E.g if you want to set the RTS line, wait until 
 *                the thread ends or stop it, make your changes and restart
 *                it. Otherwhise it could cause to mystic errors. 
 *
 *  \param  hPort    = Handle to an open Comport.
 *  \param  fpOnChar = Function pointer which would be called if a char
 *                     was received. The function should be defined as 
 *                     follow in your application: 
 *                     static void OnCharRecEvent( char cChar, LPARAM lParam )
 *                     cChar  = Char which was received
 *                     lParam = The parameter which was defined in this 
 *                              function call.
 *  \param  lParam   = If you are programming in C++ you should fill the
 *                     parameter with the known 'this' operator of your 
 *                     class so you can access all member function in 
 *                     the static callback OnCharRecEvent() function. To get 
 *                     instant access to your class member you have to do
 *                     something like this: Class foo *pbar = (foo*)lParam; 
 *                     You can fill the LPARAM also with a HWnd handle to 
 *                     the procedure. If you dont need it, fill it with NULL.
 *  \return            Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_ReaderThreadStart( HANDLE hPort, 
                                                     fpChar fpOnChar,  
                                                     LPARAM lParam = NULL );


// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*! \brief  Writes data to specified comport in a thread. If a byte is written
 *          successfull the thread will throw a window message to your app.
 *          You have to take a focus on the ON_SER_CHAR_SEND message.
 *          Your function to handle the parameter should be look like the 
 *          following sample:
 *          LRESULT OnCharSend( WPARAM wParam, LPARAM lParam );
 *          wParam  = Character which was sent successfull.
 *          lParam  = Character string position.
 *          LRESULT = 0
 *          If wParam and lParam = 0, the transmission is complete and the
 *          writer thread ends.
 *          NOTE: If the thread is running you should avoid all access to 
 *                the port. E.g if you want to set the RTS line wait until 
 *                the thread ends. Otherwhise it could cause to mystic errors. 
 *
 *  \param  *cData   = Pointer to data for writing. It is essential, that the
 *                     data is available until the thread ends !!. Be carefull
 *                     if you use no global buffer. 
 *  \param  ulLenght = Lenght of data. Set to zero if your string is NULL 
 *                     terminated and if you want to send the complete string.
 *  \param  hPort    = Handle to an open Comport. 
 *  \param  hWnd     = Window Handle. Set to NULL if you need no windows
 *                     message notification.
 *  \return Return true if success else false. */
extern "C" DLL_IM_EXPORT bool ser_WriterThread( char           *cData,
                                                unsigned long  ulLenght, 
                                                HANDLE         hPort, 
                                                HWND           hWnd = NULL );


#endif // SERIAL_PORTS_DLL_H
